import * as Babel from "@babel/standalone";
import { importResolverPlugin, resolverId } from "./import-resolver-plugin.js";
var cases = [{
  name: "basic",
  source: 'import React from "react";',
  expected: "const React = ".concat(resolverId, "(\"react\");")
}, {
  name: "destructure",
  source: "import { has, get } from 'lodash'",
  expected: "const {\n  has\n} = ".concat(resolverId, "(\"lodash\");\n\nconst {\n  get\n} = ").concat(resolverId, "(\"lodash\");")
}, {
  name: "namespace",
  source: "import * as _ from 'lodash'",
  expected: "const _ = ".concat(resolverId, "(\"lodash\");")
}, {
  name: "combined",
  source: "import React, { Component } from 'react'",
  expected: "const React = ".concat(resolverId, "(\"react\");\n\nconst {\n  Component\n} = ").concat(resolverId, "(\"react\");")
}, {
  name: "as",
  source: "import { Component as C } from 'react'",
  expected: "const {\n  Component: C\n} = ".concat(resolverId, "(\"react\");")
}];
Babel.registerPlugin("import-resolver", importResolverPlugin);
describe("importResolverPlugin", function () {
  test.each(cases)('transforms "$name" imports', function (_ref) {
    var expected = _ref.expected,
        source = _ref.source;

    var _Babel$transform = Babel.transform(source, {
      plugins: ["import-resolver"]
    }),
        code = _Babel$transform.code;

    expect(code).toEqual(expected);
  });
});