import _extends from '@babel/runtime/helpers/extends';
import _objectWithoutProperties from '@babel/runtime/helpers/objectWithoutProperties';
import _classCallCheck from '@babel/runtime/helpers/classCallCheck';
import _createClass from '@babel/runtime/helpers/createClass';
import _possibleConstructorReturn from '@babel/runtime/helpers/possibleConstructorReturn';
import _getPrototypeOf from '@babel/runtime/helpers/getPrototypeOf';
import _inherits from '@babel/runtime/helpers/inherits';
import _assertThisInitialized from '@babel/runtime/helpers/assertThisInitialized';
import _defineProperty from '@babel/runtime/helpers/defineProperty';
import { object, func, string, bool } from 'prop-types';
import { useMemo, useRef, createElement, useEffect, cloneElement, createRef, PureComponent } from 'react';
import { isValidElementType } from 'react-is';
import { availablePlugins, registerPlugin, transform } from '@babel/standalone';
import merge from 'deepmerge';
import isEqual from 'react-fast-compare';

var resolverId = "__resolverFn__";

var createVariableDeclaration = function createVariableDeclaration(t, specifier, source) {
  return t.variableDeclaration("const", [t.variableDeclarator(t.identifier(specifier.local.name), t.callExpression(t.identifier(resolverId), [t.stringLiteral(source.value)]))]);
};

var createDeclaration = function createDeclaration(t, specifier, source) {
  return t.variableDeclaration("const", [t.variableDeclarator(t.identifier(specifier.local.name), t.memberExpression(t.callExpression(t.identifier(resolverId), [t.stringLiteral(source.value)]), t.identifier(specifier.imported.name)))]);
};

var importResolverPlugin = function importResolverPlugin(_ref) {
  var t = _ref.types;
  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        var declarations = path.node.specifiers.reduce(function (acc, specifier) {
          if (t.isImportDefaultSpecifier(specifier) || t.isImportNamespaceSpecifier(specifier)) {
            acc.push(createVariableDeclaration(t, specifier, path.node.source));
          }

          if (t.isImportSpecifier(specifier)) {
            acc.push(createDeclaration(t, specifier, path.node.source));
          }

          return acc;
        }, []);
        path.replaceWithMultiple(declarations);
      }
    }
  };
};

var exportToIifePlugin = function exportToIifePlugin(_ref) {
  var t = _ref.types;
  return {
    visitor: {
      ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
        var program = path.find(function (parentPath) {
          return parentPath.isProgram();
        });
        var declaration = path.node.declaration;

        if (t.isClassDeclaration(declaration)) {
          path.replaceWith(declaration);

          if (declaration.id === null) {
            throw new Error("`export default class {}` is not supported");
          }

          program.pushContainer("body", [t.returnStatement(t.identifier(declaration.id.name))]);
          return;
        }

        if (t.isIdentifier(declaration)) {
          program.pushContainer("body", [t.returnStatement(t.identifier(declaration.name))]);
          path.remove();
        }
      },
      Program: {
        exit: function exit(path) {
          path.stop();
        }
      }
    }
  };
};

// @ts-ignore There are no types for "@babel/standalone" :(

var registerBabelPlugin = function registerBabelPlugin(pluginName, pluginFunction) {
  if (!availablePlugins[pluginName]) {
    registerPlugin(pluginName, pluginFunction);
  }
};

registerBabelPlugin("export-to-iife", exportToIifePlugin);
registerBabelPlugin("import-resolver", importResolverPlugin);
var useBabelConfig = function useBabelConfig(config) {
  var latestConfig = useRef();
  var mergedConfig = useRef({});

  if (!isEqual(latestConfig.current, config)) {
    latestConfig.current = config;
    mergedConfig.current = merge({
      plugins: ["import-resolver", "export-to-iife"],
      presets: ["react"]
    }, config);
  }

  return mergedConfig.current;
};
var useBabelTransform = function useBabelTransform(config, source) {
  try {
    var mergedConfig = useBabelConfig(config);
    var transformed = useMemo(function () {
      return transform(source, mergedConfig).code;
    }, [source, mergedConfig]);
    return {
      error: null,
      transformed: transformed
    };
  } catch (error) {
    return {
      error: error,
      transformed: ""
    };
  }
};

var hmrId = 0;

if (process.env.NODE_ENV !== "production") {
  if (module.hot) {
    module.hot.addStatusHandler(function (status) {
      if (status === "idle") {
        hmrId += 1;
      }
    });
  }
}

var useHMRId = function useHMRId(enabled) {
  return enabled ? hmrId : 0;
};

/**
 * Evaluates passed source code, uses passed resolver to resolve imports.
 */

var useEvalSourceCode = function useEvalSourceCode(hot, source, resolver) {
  var hmrId = useHMRId(hot);
  return useMemo(function () {
    return new Function(resolverId, source)(resolver);
  }, [hmrId, source, resolver]);
};
/**
 * Creates a valid React Component from source.
 */

var useCreateComponentFromSource = function useCreateComponentFromSource(babelConfig, hot, source, resolver) {
  var _useBabelTransform = useBabelTransform(babelConfig, source),
      error = _useBabelTransform.error,
      transformed = _useBabelTransform.transformed;

  var component = useEvalSourceCode(hot, transformed, resolver); // We have to rethrow error because in other case we will break hooks stack

  if (error) {
    throw error;
  }

  if (isValidElementType(component)) {
    return component;
  }

  throw new Error("Render: your source should have a default export with a React component");
};

var useRenderElement = function useRenderElement(options) {
  var babelConfig = options.babelConfig,
      source = options.source,
      resolver = options.resolver,
      unstable_hot = options.unstable_hot,
      rest = _objectWithoutProperties(options, ["babelConfig", "source", "resolver", "unstable_hot"]);

  var latestElement = useRef(null);

  try {
    var component = useCreateComponentFromSource(babelConfig, unstable_hot, source, resolver);
    latestElement.current = createElement(component, rest);
    return {
      element: latestElement.current,
      error: null
    };
  } catch (error) {
    return {
      element: latestElement.current,
      error: error
    };
  }
};

var ComponentRender = function ComponentRender(props) {
  var fallbackElement = props.fallbackElement,
      runtimeError = props.runtimeError,
      onRender = props.onRender;

  var _useRenderElement = useRenderElement(props),
      element = _useRenderElement.element,
      compilerError = _useRenderElement.error;

  var error = runtimeError || compilerError;
  useEffect(function () {
    if (!error) {
      fallbackElement.current = cloneElement(element);
    }

    if (onRender) {
      onRender(error);
    }
  });
  return error ? fallbackElement.current : element;
};

var SourceRender =
/*#__PURE__*/
function (_React$PureComponent) {
  _inherits(SourceRender, _React$PureComponent);

  function SourceRender() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, SourceRender);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(SourceRender)).call.apply(_getPrototypeOf2, [this].concat(args)));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "fallbackElement", createRef());

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "state", {
      error: null,
      source: ''
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "handleResolve", function (importName) {
      var _ref = _this.props,
          resolver = _ref.resolver,
          resolverContext = _ref.resolverContext;
      return resolver(importName, resolverContext);
    });

    return _this;
  }

  _createClass(SourceRender, [{
    key: "render",
    value: function render() {
      var _ref2 = this.props,
          resolver = _ref2.resolver,
          resolverContext = _ref2.resolverContext,
          rest = _objectWithoutProperties(_ref2, ["resolver", "resolverContext"]);

      var error = this.state.error;
      return createElement(ComponentRender, _extends({}, rest, {
        fallbackElement: this.fallbackElement,
        resolver: this.handleResolve,
        runtimeError: error
      }));
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(props, state) {
      return {
        source: props.source,
        error: props.source === state.source ? state.error : null
      };
    }
  }, {
    key: "getDerivedStateFromError",
    value: function getDerivedStateFromError(error) {
      return {
        error: error
      };
    }
  }]);

  return SourceRender;
}(PureComponent);

_defineProperty(SourceRender, "propTypes", {
  babelConfig: object,
  onError: func,
  resolver: func.isRequired,
  resolverContext: object,
  source: string.isRequired,
  unstable_hot: bool
});

_defineProperty(SourceRender, "defaultProps", {
  babelConfig: {},
  renderHtml: true,
  resolverContext: {},
  unstable_hot: false
});

export default SourceRender;
